<?php

/**
 * IPC_MQTT class is MQTT client wrapper
 *
 * @author Marcin Rabczynski <marcin.rabczynski@gigaset.com>
 * @package Maxwell-webui
 * @created 09.2016
 */
class IPC_MQTT implements IPC_Interface
{
    private static $request;
    private static $message;
    private static $instance = null;
    /*
     * @var Mosquitto\Client $connection
     */
    private $connection;

    public static function getInstance()
    {
        try
        {
            if (self::$instance === null)
            {
                self::$instance = new IPC();
            }
            return self::$instance;
        }
        catch (Exception $ex)
        {
            $exc = new errorexceptionhandler();
            $exc->getException($ex);

            return 0;
        }
    }

    public function sendMessageToProperty($param, $method = 'getV', $value = null)
    {
        $message = array('method' => $method, 'param' => $param, 'field' => PropertyService::getField()->webui);

        return $this->sendMessage($param, $method, $value, $message);
    }

    public function sendMessageToState($param, $method = 'getV', $value = null)
    {
        $message = array('method' => $method, 'param' => $param);

        return $this->sendMessage($param, $method, $value, $message);
    }

    public function sendMessageToAudio($value)
    {
        $topicPublish = 'web-ui/audio';

        return $this->sendCustomMessage($topicPublish, $value);
    }

    public function sendMessageToPhoneUI($value)
    {
        $topicPublish = 'web-ui/phone-ui';

        return $this->sendCustomMessage($topicPublish, json_encode($value));
    }

    /**
     * @param $topicPublish string
     * @param $value string
     * @return bool|object
     */
    public function sendCustomMessage($topicPublish, $value)
    {
        $responseMessage = $this->publish($topicPublish, $value);
        if (isset($responseMessage->result))
        {
            $responseMessage = $responseMessage->result;
        }

        return $responseMessage;
    }

    private function connect()
    {
        try
        {
            $this->connection = new Mosquitto\Client(null, true);
            $this->connection->connect(IPCHOST, 1883, 60);
        }
        catch (Exception $ex)
        {
            $exc = new errorexceptionhandler();
            $exc->getException($ex);

            return false;
        }
    }

    private function subscribe($_topic)
    {
        try
        {
            if(self::$instance === null)
            {
                self::getInstance();
            }

            $this->connect();
            $this->connection->onMessage(function ($_message)
            {
                $this->messageHandler($_message);
            });
            $this->connection->subscribe($_topic, 1);
            $this->connection->loopForever(5000);
        }
        catch(Exception $ex)
        {
            $exc = new errorexceptionhandler();
            $exc->getException($ex);

            return false;
        }
    }

    private function publish($_topicPublish, $_message)
    {
        try
        {

            if(self::$instance === null)
            {
                self::getInstance();
            }

            $this->connect();
            self::$request = $_message;
            $this->connection->publish($_topicPublish, self::$request);
            loger::debug($_topicPublish, __FILE__, __LINE__);
            loger::debug(self::$request, __FILE__, __LINE__);

            $this->connection->disconnect();
            return true;
        }
        catch (Exception $ex)
        {
            $exc = new errorexceptionhandler();
            $exc->getException($ex);

            return false;
        }
    }

    private function sendMessage($param, $method = 'getV', $value = null, $message)
    {
        $topicPublish = 'web-ui/prop-service';
        $topicSubscribe = 'prop-service/web-ui';

        if ($value !== null)
        {
            if ($method === 'getV' || $method === 'getColsL')
            {
                $message['fields'] = $value;
            } else if ($method === 'setV')
            {
                $message['value'] = $value;
            } else if ($method === 'setVList')
            {
                $message['list'] = $value;
            } else if ($method === 'makeBackup')
            {
                $message['allSettings'] = $value;
            }
        }
        $responseMessage = $this->publishWait($topicPublish, $topicSubscribe, $message);

        if (isset($responseMessage->result))
        {
            $responseMessage = $responseMessage->result;
        }

        return $responseMessage;
    }

    private function publishWait($_topicPublish, $_topicSubscribe, $_message)
    {
        try
        {
            if(self::$instance === null)
            {
                self::getInstance();
            }
            //lock here
            $filename = '/var/volatile/webui-lock';
            $fp = fopen($filename, "w+");
            if (flock($fp, LOCK_EX))
            {
                fwrite($fp, "Locked access to property service!!!");
            }
            //
            $this->connect();
            /*
            * @var Mosquitto\Client $connection
            */
            $this->connection->onMessage(function ($_message)
            {
                loger::debug($_message, __FILE__, __LINE__);
                $this->connection->exitLoop();
                $this->messageHandler($_message);
            });
            $this->connection->subscribe($_topicSubscribe, 1);
            self::$request = json_encode($_message);
            self::$message = "";
            $this->connection->publish($_topicPublish, self::$request);
            //wait maximum 10s for answer from PropertyService
            $this->connection->loopForever(10000);
            if (self::$message == "")
            {
              loger::debug('Response message not recived', __FILE__, __LINE__);
            }
            $this->connection->disconnect();

            //unlock here
            flock($fp, LOCK_UN);
            fclose($fp);
            return self::$message;
        }
        catch (Exception $ex)
        {
            $exc = new errorexceptionhandler();
            $exc->getException($ex);

            return false;
        }
    }

    private function messageHandler($_message)
    {
        self::$message = json_decode(trim($_message->payload));

        return $_message->payload;
    }
}
